#include <Redacted3D/engine/world.h>
#include <Redacted3D/types/entity/3D/moby.h>


int DataModel::getEntityCount() const {
    return this->children.size();
}

int DataModel::getMobyCount() const {
    int count;
    for (auto& e : this->children)
        if (auto* m = dynamic_cast<Moby*>(e))
            count++;
    return count;
}

void World::setAmbientLightColor(GLfloat red, GLfloat green, GLfloat blue) {
    GLfloat ambient[4] = {red, green, blue, 0.0f};
    globalLightColor.x = red; globalLightColor.y = green; globalLightColor.z = blue;
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
}

Vector3 World::getAmbientLightColor() {
    return globalLightColor;
}

void World::setGlobalFogColor(GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha) {
    GLfloat fog[4] = {red, green, blue, alpha};
    glFogfv(GL_FOG_COLOR, fog);
    globalFogColor.x = red; globalFogColor.y = green; globalFogColor.z = blue; globalFogColor.w = alpha;
}

Vector4 World::getGlobalFogColor() {
    return globalFogColor;
}

void World::setGlobalFogRange(GLfloat start, GLfloat end) {
    glFogf(GL_FOG_START, start);
    glFogf(GL_FOG_END, end);
    globalFogRange = {start, end};

}

Vector2 World::getGlobalFogRange() {
    return globalFogRange;
}

void World::setGlobalFogMode(GLenum mode) {
    glFogi(GL_FOG_MODE, mode);
    globalFogMode = mode;
}

GLenum World::getGlobalFogMode() const {
    return globalFogMode;
}

GLfloat World::getGlobalFogDensity() const {
    return globalFogDensity;
}

void World::setGlobalFogDensity(GLfloat density) {
    glFogf(GL_FOG_DENSITY, density);
    globalFogDensity = density;
}

Camera *World::getActiveCamera() {
    return activeCamera;
}

void World::setActiveCamera(Camera *camera) {
    activeCamera = camera;
}
